@@ -1,8 +1,8 @@
 
-__version__ = "0.11.7"
+__version__ = "0.12"
 __version_vcs_info__ = {
     'branch': 'master',
-    'revid': '0baa648',
-    'revno': 1933}
+    'revid': '81cf516',
+    'revno': 2093}
 __version_user_email__ = "saxtouri@admin.grnet.gr"
 __version_user_name__ = "Stavros Sachtouris"
